@@ -30,20 +30,21 @@
 #' Retrieve individual pulses, flightlines or scanlines
 #'
 #' Retrieve each individual pulse, individual flightline or individual scanline and assigns a number
-#' to each point.
+#' to each point. The LAS object must be properly populated according to LAS specifications otherwise
+#' users could find unexpected outputs.
 #'
 #' \describe{
 #' \item{\code{laspulse}}{Retrieves each individual pulse. It uses GPS time. An attribute
-#' '\code{pulseID}' is added in the \code{LAS} object}
+#' \code{pulseID} is added in the \code{LAS} object}
 #' \item{\code{lasscanline}}{Retrieves each individual scanline. When data are sampled according to a
 #' saw-tooth pattern (oscillating mirror), a scanline is one line, or row of data. The function relies
-#' on the GPS field time to order the data. Then, the 'ScanDirectionFlag' attribute (when available)
-#' is used to retrieve each scanline. An attribute '\code{scanlineID}' is added in the \code{LAS} object}
+#' on the GPS field time to order the data. Then, the \code{ScanDirectionFlag} attribute is used to
+#' retrieve each scanline. An attribute \code{scanlineID} is added in the \code{LAS} object}
 #' \item{\code{lasflightline}}{Retrieves each individual flightline. It uses GPS time. In a
 #' continuous dataset, once points are ordered by GPS time, the time between two consecutive points
 #' does not exceed a few milliseconds. If the time between two consecutive points is too long it means
 #' that the second point is from a different flightline. The default threshold is 30 seconds.
-#' An attribute '\code{flightlineID}' is added in the \code{LAS} object.}
+#' An attribute \code{flightlineID} is added in the \code{LAS} object.}
 #' }
 #'
 #' @param las A LAS object
@@ -68,11 +69,19 @@ laspulse = function(las)
   stopifnotlas(las)
 
   if (!"gpstime" %in% names(las@data))
-    stop("No gpstime attribute found. Pulses cannot be computed from this file.")
+    stop("No 'gpstime' attribute found. Pulse IDs cannot be computed from this object")
+
+  if (all(las@data[["gpstime"]] == 0))
+  {
+    warning("'gpstime' attribute is populated with 0 only. A single ID has been returned for each point.")
+    las@data[["pulseID"]] <- 1L
+    return(las)
+  }
 
   gpstime <- NULL
   data.table::setorder(las@data, gpstime)
-  las@data[["pulseID"]] <- .lagisdiff(las@data[["gpstime"]])
+  pulseID <- .lagisdiff(las@data[["gpstime"]])
+  las@data[["pulseID"]] <- pulseID
   return(las)
 }
 
@@ -85,11 +94,19 @@ lasflightline = function(las, dt = 30)
   assert_all_are_non_negative(dt)
 
   if (!"gpstime" %in% names(las@data))
-    stop("No gpstime attribute found. Flightlines cannot be computed from this file.")
+    stop("No gpstime attribute found. Flightlines cannot be computed from this object")
+
+  if (all(las@data[["gpstime"]] == 0))
+  {
+    warning("'gpstime' attribute is populated with 0 only. A single ID has been returned for each point.")
+    las@data[["flightlineID"]] <- 1L
+    return(las)
+  }
 
   gpstime <- NULL
   data.table::setorder(las@data, gpstime)
-  las@data[["flightlineID"]] <- .lagissup(las@data[["gpstime"]], dt)
+  flightlineID <- .lagissup(las@data[["gpstime"]], dt)
+  las@data[["flightlineID"]] <- flightlineID
   return(las)
 }
 
@@ -100,19 +117,20 @@ lasscanline = function(las)
   stopifnotlas(las)
 
   if (!"gpstime" %in% names(las@data))
-    stop("No gpstime attribute found. Scanlines cannot be computed from this object")
+    stop("No 'gpstime' attribute found. Scanlines IDs cannot be computed from this object")
 
   if (!"ScanDirectionFlag" %in% names(las@data))
-    stop("No ScanDirectionFlag field found. Scanlines cannot be computed from this object")
+    stop("No 'ScanDirectionFlag' attribute found. Scanlines cannot be computed from this object")
 
-  values <- unique(las@data$ScanDirectionFlag)
+  values <- unique(las@data[["ScanDirectionFlag"]])
 
   if (!all(sort(values) == c(0L,1L)))
-    stop("ScanDirectionFlag field is not properly populated according to LAS specifications. Scanlines cannot be computed from this object'")
+    stop("'ScanDirectionFlag' attribute is not properly populated according to LAS specifications. Scanlines IDs cannot be computed from this object'")
 
   gpstime <- NULL
   data.table::setorder(las@data, gpstime)
-  las@data[["scanlineID"]] <- .lagisdiff(las@data[["ScanDirectionFlag"]])
+  scanlineID <- .lagisdiff(las@data[["ScanDirectionFlag"]])
+  las@data[["scanlineID"]] <- scanlineID
   return(las)
 }
 
