@@ -115,7 +115,7 @@ lassnags.LAScatalog = function(las, algorithm, attribute = "snagCls")
   options <- list(need_buffer = TRUE, drop_null = TRUE, need_output_file = TRUE)
   output  <- catalog_apply(las, lassnags,  algorithm = algorithm, .options = options)
   output  <- unlist(output)
-  ctg     <- catalog(output)
+  ctg     <- readLAScatalog(output)
 
   opt_copy(ctg) <- las
   return(ctg)
