@@ -25,12 +25,16 @@
 #
 # ===============================================================================
 
+# ==== STANDARD ====
+
 #' Predefined standard metrics functions
 #'
-#' Predefined functions usable in \link{grid_metrics}, \link{grid_hexametrics}, \link{lasmetrics}, \link{tree_metrics},
-#' and their convenient shortcuts. The philosophy of the \code{lidR} package is to provide an easy way
-#' to compute user-defined metrics rather than to provide them. However, for efficiency and to save time, a set of
-#' standard metrics has been predefined. To use these functions please read the details and examples sections.
+#' Predefined functions computable at pixel level (\link{grid_metrics}), hexagonal cell level
+#' (\link{grid_hexametrics}), point cloud level (\link{lasmetrics}), tree level (\link{tree_metrics})
+#' and voxel level (\link{grid_metrics3d}). Each function comes with a convenient shortcuts for lazy
+#' coding. The \code{lidR} package aims to provide an easy way to compute user-defined metrics rather
+#' than to provide them. However, for efficiency and to save time, a set of standard metrics has been
+#' predefined (see details.
 #'
 #' The function names, their parameters and the output names of the metrics rely on a nomenclature chosen for brevity:
 #' \itemize{
@@ -52,10 +56,16 @@
 #' intensity at a given percentile of height.\cr\cr
 #' Each function has a convenient associated variable. It is the name of the function, with a
 #' dot before the name. This enables the function to be used without writing parameters. The cost
-#' of such a feature is inflexibility. It corresponds to a predefined behavior (see examples)\cr\cr
-#' \code{stdtreemetrics} is a special function that works with \link{tree_metrics}. Actually,
+#' of such a feature is inflexibility. It corresponds to a predefined behavior (see examples)\cr
+#' \describe{
+#' \item{\code{stdmetrics}}{is a combinaison of \code{stdmetrics_ctrl} + \code{stdmetrics_z} +
+#' \code{stdmetrics_i} +  \code{stdmetrics_rn}}
+#' \item{\code{stdtreemetrics}}{is a special function that works with \link{tree_metrics}. Actually,
 #' it won't fail with other functions but the output makes more sense if computed at the
-#' individual tree level.
+#' individual tree level.}
+#' \item{\code{stdshapemetrics}}{is a set of eigenvalue based feature decribed in Lucas et al, 2019
+#' (see references).}
+#' }
 #'
 #' @param x,y,z,i Coordinates of the points, Intensity
 #' @param rn,class ReturnNumber, Classification
@@ -67,16 +77,16 @@
 #' las = readLAS(LASfile, select = "*")
 #'
 #' # All the predefined metrics
-#' m1 = grid_metrics(las, stdmetrics(X,Y,Z,Intensity,ReturnNumber,Classification,dz=1))
+#' m1 = grid_metrics(las, ~stdmetrics(X,Y,Z,Intensity,ReturnNumber,Classification,dz=1))
 #'
 #' # Convenient shortcut
 #' m2 = grid_metrics(las, .stdmetrics)
 #'
 #' # Basic metrics from intensities
-#' m3 = grid_metrics(las, stdmetrics_i(Intensity))
+#' m3 = grid_metrics(las, ~stdmetrics_i(Intensity))
 #'
 #' # All the metrics from intensities
-#' m4 = grid_metrics(las, stdmetrics_i(Intensity, Z, Classification, ReturnNumber))
+#' m4 = grid_metrics(las, ~stdmetrics_i(Intensity, Z, Classification, ReturnNumber))
 #'
 #' # Convenient shortcut for the previous example
 #' m5 = grid_metrics(las, .stdmetrics_i)
@@ -116,7 +126,7 @@
 #'   return( c(metrics, stdmetrics_z(z)) )
 #' }
 #'
-#' m10 = grid_metrics(las, myMetrics(Z, Intensity, ReturnNumber))
+#' m10 = grid_metrics(las, ~myMetrics(Z, Intensity, ReturnNumber))
 #'
 #' # Users can write their own convenient shorcuts like this:
 #' .myMetrics = ~myMetrics(Z, Intensity, ReturnNumber)
@@ -142,194 +152,6 @@ stdmetrics = function(x, y, z, i, rn, class, dz = 1, th = 2)
   return(metrics)
 }
 
-#' @rdname stdmetrics
-#' @export
-.stdmetrics = ~stdmetrics(X,Y,Z,Intensity, ReturnNumber, Classification, dz = 1, th = 2)
-
-#' Gap fraction profile
-#'
-#' Computes the gap fraction profile using the method of Bouvier et al. (see reference)
-#'
-#' The function assesses the number of laser points that actually reached the layer
-#' z+dz and those that passed through the layer [z, z+dz]. By definition the layer 0
-#' will always return 0 because no returns pass through the ground. Therefore, the layer 0 is removed
-#' from the returned results.
-#'
-#' @param z vector of positive z coordinates
-#' @param dz numeric. The thickness of the layers used (height bin)
-#' @param z0 numeric. The bottom limit of the profile
-#' @return A data.frame containing the bin elevations (z) and the gap fraction for each bin (gf)
-#' @examples
-#' z = c(rnorm(1e4, 25, 6), rgamma(1e3, 1, 8)*6, rgamma(5e2, 5,5)*10)
-#' z = z[z<45 & z>0]
-#'
-#' hist(z, n=50)
-#'
-#' gapFraction = gap_fraction_profile(z)
-#'
-#' plot(gapFraction, type="l", xlab="Elevation", ylab="Gap fraction")
-#' @references Bouvier, M., Durrieu, S., Fournier, R. a, & Renaud, J. (2015).  Generalizing predictive models of forest inventory attributes using an area-based approach with airborne las data. Remote Sensing of Environment, 156, 322-334. http://doi.org/10.1016/j.rse.2014.10.004
-#' @seealso \link[lidR:LAD]{LAD}
-#' @export gap_fraction_profile
-gap_fraction_profile = function(z, dz = 1, z0 = 2)
-{
-
-  bk = seq(floor((min(z) - z0)/dz)*dz + z0, ceiling((max(z) - z0)/dz)*dz + z0, dz)
-
-  histogram = graphics::hist(z, breaks = bk, plot = F)
-  h = histogram$mids
-  p = histogram$counts/sum(histogram$counts)
-
-  p = c(p, 0)
-
-  cs = cumsum(p)
-  i = data.table::shift(cs)/cs
-  i[is.na(i)] = 0
-
-  i[is.nan(i)] = NA
-
-  z = h #[-1]
-  i = i[-length(i)] #[-c(1, length(i))]
-
-  return(data.frame(z = z[z > z0], gf = i[z > z0]))
-}
-
-#' Leaf area density
-#'
-#' Computes a leaf area density profile based on the method of Bouvier et al. (see reference)
-#'
-#' The function assesses the number of laser points that actually reached the layer
-#' z+dz and those that passed through the layer [z, z+dz] (see \link[lidR:gap_fraction_profile]{gap_fraction_profile}).
-#' Then it computes the log of this quantity and divides it by the extinction coefficient k as described in Bouvier
-#' et al. By definition the layer 0 will always return infinity because no returns pass through
-#' the ground. Therefore, the layer 0 is removed from the returned results.
-#'
-#' @param z vector of positive z coordinates
-#' @param dz numeric. The thickness of the layers used (height bin)
-#' @param k numeric. is the extinction coefficient
-#' @param z0 numeric. The bottom limit of the profile
-#' @return A data.frame containing the bin elevations (z) and leaf area density for each bin (lad)
-#' @examples
-#' z = c(rnorm(1e4, 25, 6), rgamma(1e3, 1, 8)*6, rgamma(5e2, 5,5)*10)
-#' z = z[z<45 & z>0]
-#'
-#' lad = LAD(z)
-#'
-#' plot(lad, type="l", xlab="Elevation", ylab="Leaf area density")
-#' @references Bouvier, M., Durrieu, S., Fournier, R. a, & Renaud, J. (2015).  Generalizing predictive models of forest inventory attributes using an area-based approach with airborne las data. Remote Sensing of Environment, 156, 322-334. http://doi.org/10.1016/j.rse.2014.10.004
-#' @seealso \link[lidR:gap_fraction_profile]{gap_fraction_profile}
-#' @export LAD
-LAD = function(z, dz = 1, k = 0.5, z0 = 2) # (Bouvier et al. 2015)
-{
-	ld = gap_fraction_profile(z, dz, z0)
-
-	if (anyNA(ld))
-	  return(NA_real_)
-
-	lad = ld$gf
-	lad = -log(lad)/(k*dz)
-
-	lad[is.infinite(lad)] = NA
-
-	lad = lad
-
-	return(data.frame(z = ld$z, lad))
-}
-
-#' Normalized Shannon diversity index
-#'
-#' A normalized Shannon vertical complexity index. The Shannon diversity index is a measure for
-#' quantifying diversity and is based on the number and frequency of species present. This index,
-#' developed by Shannon and Weaver for use in information theory, was successfully transferred
-#' to the description of species diversity in biological systems (Shannon 1948). Here it is applied
-#' to quantify the diversity and the evenness of an elevational distribution of las points. It
-#' makes bins between 0 and the maximum elevation. If there are negative values the function
-#' returns NA.
-#'
-#' @param z vector of positive z coordinates
-#' @param by numeric. The thickness of the layers used (height bin)
-#' @param zmax numeric. Used to turn the function entropy to the function \link[lidR:VCI]{VCI}.
-#' @seealso
-#' \link[lidR:VCI]{VCI}
-#' @examples
-#' z = runif(10000, 0, 10)
-#'
-#' # expected to be close to 1. The highest diversity is given for a uniform distribution
-#' entropy(z, by = 1)
-#'
-#'  z = runif(10000, 9, 10)
-#'
-#' # Must be 0. The lowest diversity is given for a unique possibility
-#' entropy(z, by = 1)
-#'
-#' z = abs(rnorm(10000, 10, 1))
-#'
-#' # expected to be between 0 and 1.
-#' entropy(z, by = 1)
-#' @references
-#' Pretzsch, H. (2008). Description and Analysis of Stand Structures. Springer Berlin Heidelberg. http://doi.org/10.1007/978-3-540-88307-4 (pages 279-280)
-#' Shannon, Claude E. (1948), "A mathematical theory of communication," Bell System Tech. Journal 27, 379-423, 623-656.
-#' @return A number between 0 and 1
-#' @export entropy
-entropy = function(z, by = 1, zmax = NULL)
-{
-  # Fixed entropy (van Ewijk et al. (2011)) or flexible entropy
-  if (is.null(zmax))
-	  zmax = max(z)
-
-	# If zmax < 3 it is meaningless to compute entropy
-	if (zmax < 2 * by)
-		return(NA_real_)
-
-  if (min(z) < 0)
-    return(NA_real_)
-
-	# Define the number of x meters bins from 0 to zmax (rounded to the next integer)
-	bk = seq(0, ceiling(zmax/by)*by, by)
-
-	# Compute the p for each bin
-	hist = findInterval(z, bk)
-	hist = fast_table(hist, length(bk) - 1)
-	hist = hist/sum(hist)
-
-	# Remove bins where there are no points because of log(0)
-	p    = hist[hist > 0]
-	pref = rep(1/length(hist), length(hist))
-
-	# normalized entropy
-	S = -sum(p*log(p)) / -sum(pref*log(pref))
-
-	return(S)
-}
-
-#' Vertical Complexity Index
-#'
-#' A fixed normalization of the entropy function (see references)
-#' @param z vector of z coordinates
-#' @param by numeric. The thickness of the layers used (height bin)
-#' @param zmax numeric. Used to turn the function entropy to the function vci.
-#' @return A number between 0 and 1
-#' @seealso
-#' \link[lidR:entropy]{entropy}
-#' @examples
-#' z = runif(10000, 0, 10)
-#'
-
-#' VCI(z, by = 1, zmax = 20)
-#'
-#' z = abs(rnorm(10000, 10, 1))
-#'
-#' # expected to be closer to 0.
-#' VCI(z, by = 1, zmax = 20)
-#' @references van Ewijk, K. Y., Treitz, P. M., & Scott, N. A. (2011). Characterizing Forest Succession in Central Ontario using LAS-derived Indices. Photogrammetric Engineering and Remote Sensing, 77(3), 261-269. Retrieved from <Go to ISI>://WOS:000288052100009
-#' @export VCI
-VCI = function(z, zmax, by = 1)
-{
-  z = z[z < zmax]
-
-  return(entropy(z, by, zmax))
-}
-
 #' @rdname stdmetrics
 #' @export
 stdmetrics_z = function(z, dz = 1, th = 2)
@@ -377,10 +199,6 @@ stdmetrics_z = function(z, dz = 1, th = 2)
   return(metrics)
 }
 
-#' @rdname stdmetrics
-#' @export
-.stdmetrics_z = ~stdmetrics_z(Z, dz =1)
-
 #' @rdname stdmetrics
 #' @export
 stdmetrics_i = function(i, z = NULL, class = NULL, rn = NULL)
@@ -429,10 +247,6 @@ stdmetrics_i = function(i, z = NULL, class = NULL, rn = NULL)
   metrics
 }
 
-#' @rdname stdmetrics
-#' @export
-.stdmetrics_i = ~stdmetrics_i(Intensity, Z, Classification, ReturnNumber)
-
 #' @rdname stdmetrics
 #' @export
 stdmetrics_rn = function(rn, class = NULL)
@@ -453,11 +267,6 @@ stdmetrics_rn = function(rn, class = NULL)
   return(metrics)
 }
 
-#' @rdname stdmetrics
-#' @export
-.stdmetrics_rn = ~stdmetrics_rn(ReturnNumber, Classification)
-
-
 #' @rdname stdmetrics
 #' @export
 stdmetrics_pulse = function(pulseID, rn)
@@ -479,10 +288,6 @@ stdmetrics_pulse = function(pulseID, rn)
   return(np)
 }
 
-#' @rdname stdmetrics
-#' @export
-.stdmetrics_pulse = ~stdmetrics_pulse(pulseID, ReturnNumber)
-
 #' @rdname stdmetrics
 #' @export
 stdmetrics_ctrl = function(x, y, z)
@@ -498,52 +303,129 @@ stdmetrics_ctrl = function(x, y, z)
   return(metrics)
 }
 
-#' @rdname stdmetrics
-#' @export
-.stdmetrics_ctrl = ~stdmetrics_ctrl(X, Y, Z)
-
 #' @rdname stdmetrics
 #' @export
 stdtreemetrics = function(x, y, z)
 {
-  npoints = length(x)
-  convhull.area = area_convex_hull(x,y)
-
   metrics = list(
-    npoints = npoints,
-    convhull_area = convhull.area
+    Z = max(z),
+    npoints = length(x),
+    convhull_area = round(area_convex_hull(x,y),3)
   )
 
   return(metrics)
 }
+#' @rdname stdmetrics
+#' @export
+#' @references
+#' Lucas, C., Bouten, W., Koma, Z., Kissling, W. D., & Seijmonsbergen, A. C. (2019). Identification
+#' of Linear Vegetation Elements in a Rural Landscape Using LiDAR Point Clouds. Remote Sensing, 11(3), 292.
+stdshapemetrics = function(x,y,z)
+{
+  xyz     <- cbind(x,y,z)
+  eigen   <- fast_eigen_values(xyz)
+  eigen_m <- eigen[[1]]
+  eigen_m <- as.numeric(eigen_m)
+
+  shapemetrics = list(
+    eigen_largest  = eigen_m[1],
+    eigen_medium   = eigen_m[2],
+    eigen_smallest = eigen_m[3],
+    curvature      = eigen_m[3]/(eigen_m[1] + eigen_m[2] + eigen_m[3]),
+    linearity      = (eigen_m[1] - eigen_m[2])/eigen_m[1],
+    planarity      = (eigen_m[2] - eigen_m[3])/eigen_m[1],
+    sphericity     = eigen_m[3]/eigen_m[1],
+    anisotropy     = (eigen_m[1] - eigen_m[3])/eigen_m[1],
+    horizontality  = abs(eigen[[2]][3,3])
+  )
+  return(shapemetrics)
+}
+
+stdtreeapex <- function(x,y,z)
+{
+  j <- which.max(z)
+  return(list(x.pos.t = x[j], y.pos.t = y[j]))
+}
+
+
+stdtreehullconvex = function(x,y, grp, ...)
+{
+  if (length(x) < 4)
+    return(NULL)
+
+  i = grDevices::chull(x,y)
+  i = c(i, i[1])
+  P = cbind(x[i], y[i])
+  poly = sp::Polygon(P)
+  poly = sp::Polygons(list(poly), ID = grp)
+
+  list(poly = list(poly))
+}
+
+stdtreehullconcave = function(x,y, grp, concavity, length_threshold)
+{
+  if (length(x) < 4)
+    return(NULL)
+
+  P = concaveman::concaveman(cbind(x,y), concavity, length_threshold)
+  poly = sp::Polygon(P)
+  poly = sp::Polygons(list(poly), ID = grp)
+
+  list(poly = list(poly))
+}
+
+stdtreehullbbox = function(x,y, grp, ...)
+{
+  if (length(x) < 4)
+    return(NULL)
+
+  xmin = min(x)
+  ymin = min(y)
+  xmax = max(x)
+  ymax = max(y)
+
+  x = c(xmin, xmax, xmax, xmin, xmin)
+  y = c(ymin, ymin, ymax, ymax, ymin)
+  P = cbind(x, y)
+  poly = sp::Polygon(P)
+  poly = sp::Polygons(list(poly), ID = grp)
+
+  list(poly = list(poly))
+}
+
+#' @rdname stdmetrics
+#' @export
+.stdmetrics = ~stdmetrics(X,Y,Z,Intensity, ReturnNumber, Classification, dz = 1, th = 2)
+
+#' @rdname stdmetrics
+#' @export
+.stdmetrics_z = ~stdmetrics_z(Z, dz =1)
+
+#' @rdname stdmetrics
+#' @export
+.stdmetrics_i = ~stdmetrics_i(Intensity, Z, Classification, ReturnNumber)
+
+#' @rdname stdmetrics
+#' @export
+.stdmetrics_rn = ~stdmetrics_rn(ReturnNumber, Classification)
+
+#' @rdname stdmetrics
+#' @export
+.stdmetrics_pulse = ~stdmetrics_pulse(pulseID, ReturnNumber)
+
+#' @rdname stdmetrics
+#' @export
+.stdmetrics_ctrl = ~stdmetrics_ctrl(X, Y, Z)
 
 #' @rdname stdmetrics
 #' @export
 .stdtreemetrics = ~stdtreemetrics(X, Y, Z)
 
-# canopy = canopyMatrix(x,y,z, canopyResolution)
+#' @rdname stdmetrics
+#' @export
+.stdshapemetrics = ~stdshapemetrics(X, Y, Z)
 
-# canopyq    = as.numeric(quantile(canopy, seq(0.1, 0.9, 0.2), na.rm=T))
-#
-# canopymean = mean(canopy, na.rm=T)
-# canopysd   = sd(canopy, na.rm=T)
-# canopycv   = canopysd/canopymean
-#
-# canopyq10  = canopyq[1]
-# canopyq30  = canopyq[2]
-# canopyq50  = canopyq[3]
-# canopyq70  = canopyq[4]
-# canopyq90  = canopyq[5]
-#
-# canopyfd   = fractal.dimension(canopy)
-#
-# perc.canopy.na = sum(is.na(canopy))/length(canopy) * 100
-#
-# CC2  = sum(canopy > 2, na.rm=T)/length(canopy)*100
-# CC10 = sum(canopy > 10, na.rm=T)/length(canopy)*100
-# CC20 = sum(canopy > 20, na.rm=T)/length(canopy)*100
-# CC30 = sum(canopy > 30, na.rm=T)/length(canopy)*100
-# CC40 = sum(canopy > 40, na.rm=T)/length(canopy)*100
+# ===== OTHER =====
 
 #' Rumple index of roughness
 #'
@@ -639,69 +521,189 @@ rumple_index.numeric <- function(x, y = NULL, z = NULL, ...)
   return(rumple)
 }
 
-# fractal_dimension
-#
-# Computes the fractal dimension of a surface. The fractal dimension is a measure of roughness.
-#
-# Fractal dimension computes the roughness based on the box counting method (see Taud and Parrot).
-# If the input has an NA value, it returns NA. If the input is too small it returns NA.
-# If the input matrix is not square, the function cuts the input matrix to create a square matrix.
-# @param mtx numeric matrix that is the representation of a surface model.
-# @return numeric. A number between 0 and 3, with 3 being the dimension of a volume.
-# @references Taud, H., & Parrot, J.-F. (2005). Mesure de la rugosite des MNT a l'aide de la dimension fractale. Geomorphologie : Relief, Processus, Environnement, 4, 327-338. http://doi.org/10.4000/geomorphologie.622
-# @examples
-# mtx = matrix(runif(100), 10, 10)
-# fractal_dimension(mtx)
-# @export fractal_dimension
-# fractal_dimension = function(x, plot = FALSE)
-# {
-#   if (is(x, "RasterLayer"))
-#     x = raster::as.matrix(x)
-#   else if (!is.matrix(x))
-#     stop("fractal_dimension: mtx should be a RasterLayer or a Matrix")
-#
-#   mtx = x
-#
-#   if ( sum(is.na(mtx)) > 0 )
-#     return(NA_real_)
-#
-#   size = min(dim(mtx))
-#
-#   if ( size < 6)
-#     return(NA_real_)
-#
-#   size = ifelse(size %% 2 == 0, size, size - 1)
-#
-#   mtx = mtx[1:size, 1:size]
-#
-#   q = 1:size
-#   q = q[size %% q == 0]
-#
-#   if (length(q) < 3)
-#     return(as.numeric(NA))
-#
-#   nbbox = sapply(q, countBox, mtx = mtx)
-#
-#   lm = stats::lm(log(nbbox) ~ log(q))
-#
-#   if (plot)
-#   {
-#     graphics::plot(log(nbbox) ~ log(q))
-#     graphics::abline(lm)
-#   }
-#
-#   return(abs(as.numeric(stats::coefficients(lm)[2])))
-# }
-#
-# countBox = function(q, mtx)
-# {
-# 	  rg  <- (row(mtx) - 1) %/% q + 1
-#     cg  <- (col(mtx) - 1) %/% q + 1
-#     rci <- (rg - 1) * max(cg) + cg
-#     N   <- prod(dim(mtx))/(q^2)
-#
-# 	  clip = lapply(1:N, function(x) mtx[rci == x])
-# 	  box = sapply(clip, max)/q
-#
-# 	  return(sum(box))
-# }
+#' Gap fraction profile
+#'
+#' Computes the gap fraction profile using the method of Bouvier et al. (see reference)
+#'
+#' The function assesses the number of laser points that actually reached the layer
+#' z+dz and those that passed through the layer [z, z+dz]. By definition the layer 0
+#' will always return 0 because no returns pass through the ground. Therefore, the layer 0 is removed
+#' from the returned results.
+#'
+#' @param z vector of positive z coordinates
+#' @param dz numeric. The thickness of the layers used (height bin)
+#' @param z0 numeric. The bottom limit of the profile
+#' @return A data.frame containing the bin elevations (z) and the gap fraction for each bin (gf)
+#' @examples
+#' z = c(rnorm(1e4, 25, 6), rgamma(1e3, 1, 8)*6, rgamma(5e2, 5,5)*10)
+#' z = z[z<45 & z>0]
+#'
+#' hist(z, n=50)
+#'
+#' gapFraction = gap_fraction_profile(z)
+#'
+#' plot(gapFraction, type="l", xlab="Elevation", ylab="Gap fraction")
+#' @references Bouvier, M., Durrieu, S., Fournier, R. a, & Renaud, J. (2015).  Generalizing predictive models of forest inventory attributes using an area-based approach with airborne las data. Remote Sensing of Environment, 156, 322-334. http://doi.org/10.1016/j.rse.2014.10.004
+#' @seealso \link[lidR:LAD]{LAD}
+#' @export gap_fraction_profile
+gap_fraction_profile = function(z, dz = 1, z0 = 2)
+{
+  bk = seq(floor((min(z) - z0)/dz)*dz + z0, ceiling((max(z) - z0)/dz)*dz + z0, dz)
+
+  if (length(bk) <= 1)
+    return(data.frame(z = numeric(0), gf = numeric(0)))
+
+  histogram = graphics::hist(z, breaks = bk, plot = F)
+  h = histogram$mids
+  p = histogram$counts/sum(histogram$counts)
+
+  p = c(p, 0)
+
+  cs = cumsum(p)
+  i = data.table::shift(cs)/cs
+  i[is.na(i)] = 0
+
+  i[is.nan(i)] = NA
+
+  z = h #[-1]
+  i = i[-length(i)] #[-c(1, length(i))]
+
+  return(data.frame(z = z[z > z0], gf = i[z > z0]))
+}
+
+#' Leaf area density
+#'
+#' Computes a leaf area density profile based on the method of Bouvier et al. (see reference)
+#'
+#' The function assesses the number of laser points that actually reached the layer
+#' z+dz and those that passed through the layer [z, z+dz] (see \link[lidR:gap_fraction_profile]{gap_fraction_profile}).
+#' Then it computes the log of this quantity and divides it by the extinction coefficient k as described in Bouvier
+#' et al. By definition the layer 0 will always return infinity because no returns pass through
+#' the ground. Therefore, the layer 0 is removed from the returned results.
+#'
+#' @param z vector of positive z coordinates
+#' @param dz numeric. The thickness of the layers used (height bin)
+#' @param k numeric. is the extinction coefficient
+#' @param z0 numeric. The bottom limit of the profile
+#' @return A data.frame containing the bin elevations (z) and leaf area density for each bin (lad)
+#' @examples
+#' z = c(rnorm(1e4, 25, 6), rgamma(1e3, 1, 8)*6, rgamma(5e2, 5,5)*10)
+#' z = z[z<45 & z>0]
+#'
+#' lad = LAD(z)
+#'
+#' plot(lad, type="l", xlab="Elevation", ylab="Leaf area density")
+#' @references Bouvier, M., Durrieu, S., Fournier, R. a, & Renaud, J. (2015).  Generalizing predictive models of forest inventory attributes using an area-based approach with airborne las data. Remote Sensing of Environment, 156, 322-334. http://doi.org/10.1016/j.rse.2014.10.004
+#' @seealso \link[lidR:gap_fraction_profile]{gap_fraction_profile}
+#' @export LAD
+LAD = function(z, dz = 1, k = 0.5, z0 = 2) # (Bouvier et al. 2015)
+{
+  ld = gap_fraction_profile(z, dz, z0)
+
+  if (nrow(ld) <= 2)
+    return(data.frame(z = numeric(0), lad = numeric(0)))
+
+  if (anyNA(ld))
+    return(data.frame(z = numeric(0), lad = numeric(0)))
+
+  lad = ld$gf
+  lad = -log(lad)/(k*dz)
+
+  lad[is.infinite(lad)] = NA
+
+  lad = lad
+
+  return(data.frame(z = ld$z, lad = lad))
+}
+
+#' Normalized Shannon diversity index
+#'
+#' A normalized Shannon vertical complexity index. The Shannon diversity index is a measure for
+#' quantifying diversity and is based on the number and frequency of species present. This index,
+#' developed by Shannon and Weaver for use in information theory, was successfully transferred
+#' to the description of species diversity in biological systems (Shannon 1948). Here it is applied
+#' to quantify the diversity and the evenness of an elevational distribution of las points. It
+#' makes bins between 0 and the maximum elevation. If there are negative values the function
+#' returns NA.
+#'
+#' @param z vector of positive z coordinates
+#' @param by numeric. The thickness of the layers used (height bin)
+#' @param zmax numeric. Used to turn the function entropy to the function \link[lidR:VCI]{VCI}.
+#' @seealso
+#' \link[lidR:VCI]{VCI}
+#' @examples
+#' z = runif(10000, 0, 10)
+#'
+#' # expected to be close to 1. The highest diversity is given for a uniform distribution
+#' entropy(z, by = 1)
+#'
+#'  z = runif(10000, 9, 10)
+#'
+#' # Must be 0. The lowest diversity is given for a unique possibility
+#' entropy(z, by = 1)
+#'
+#' z = abs(rnorm(10000, 10, 1))
+#'
+#' # expected to be between 0 and 1.
+#' entropy(z, by = 1)
+#' @references
+#' Pretzsch, H. (2008). Description and Analysis of Stand Structures. Springer Berlin Heidelberg. http://doi.org/10.1007/978-3-540-88307-4 (pages 279-280)
+#' Shannon, Claude E. (1948), "A mathematical theory of communication," Bell System Tech. Journal 27, 379-423, 623-656.
+#' @return A number between 0 and 1
+#' @export entropy
+entropy = function(z, by = 1, zmax = NULL)
+{
+  # Fixed entropy (van Ewijk et al. (2011)) or flexible entropy
+  if (is.null(zmax))
+    zmax = max(z)
+
+  # If zmax < 3 it is meaningless to compute entropy
+  if (zmax < 2 * by)
+    return(NA_real_)
+
+  if (min(z) < 0)
+    return(NA_real_)
+
+  # Define the number of x meters bins from 0 to zmax (rounded to the next integer)
+  bk = seq(0, ceiling(zmax/by)*by, by)
+
+  # Compute the p for each bin
+  hist = findInterval(z, bk)
+  hist = fast_table(hist, length(bk) - 1)
+  hist = hist/sum(hist)
+
+  # Remove bins where there are no points because of log(0)
+  p    = hist[hist > 0]
+  pref = rep(1/length(hist), length(hist))
+
+  # normalized entropy
+  S = -sum(p*log(p)) / -sum(pref*log(pref))
+
+  return(S)
+}
+
+#' Vertical Complexity Index
+#'
+#' A fixed normalization of the entropy function (see references)
+#' @param z vector of z coordinates
+#' @param by numeric. The thickness of the layers used (height bin)
+#' @param zmax numeric. Used to turn the function entropy to the function vci.
+#' @return A number between 0 and 1
+#' @seealso
+#' \link[lidR:entropy]{entropy}
+#' @examples
+#' z = runif(10000, 0, 10)
+#'
+#' VCI(z, by = 1, zmax = 20)
+#'
+#' z = abs(rnorm(10000, 10, 1))
+#'
+#' # expected to be closer to 0.
+#' VCI(z, by = 1, zmax = 20)
+#' @references van Ewijk, K. Y., Treitz, P. M., & Scott, N. A. (2011). Characterizing Forest Succession in Central Ontario using LAS-derived Indices. Photogrammetric Engineering and Remote Sensing, 77(3), 261-269. Retrieved from <Go to ISI>://WOS:000288052100009
+#' @export VCI
+VCI = function(z, zmax, by = 1)
+{
+  z = z[z < zmax]
+  return(entropy(z, by, zmax))
+}
