@@ -27,8 +27,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 ===============================================================================
 */
 
-#include <Rcpp.h>
-#include "QuadTree.h"
+#include <RcppArmadillo.h>
 using namespace Rcpp;
 
 // [[Rcpp::export]]
@@ -78,3 +77,15 @@ NumericVector roundc(NumericVector x, int digit = 0)
   return y;
 }
 
+// [[Rcpp::export]]
+SEXP fast_eigen_values(arma::mat A)
+{
+  arma::mat coeff;
+  arma::mat score;
+  arma::vec latent;
+  arma::princomp(coeff, score, latent, A);
+  NumericMatrix eigenvalues = Rcpp::wrap(latent);
+  NumericMatrix eigencoeff = Rcpp::wrap(coeff);
+  return(List::create(_["eigen"] = eigenvalues, _["coeff"] = eigencoeff));
+}
+
